<?php
/** 
 * 
 * @version 1.0.2
 * @author Piotr Kozdęba <piotr.ko@modulesgarden.com>
 */

use Illuminate\Database\Capsule\Manager as DB;

class OPO_Database_Database
{
    private $result = false;
    private $affected_rows = 0;
    private $last_insert_id = false;
    private $error = '';
    
    
    /**
     * FUNCTION reset 
     * Prepares object for next query
     */
    private function reset()
    {
        $this->result = false;
        $this->affected_rows = 0;
        $this->last_insert_id = false;
        $this->error = '';
    }
    
    
    /** 
     * FUNCTION query 
     * Query database
     * @param string $query - a query to execute 
     * @return object An object of this class 
     */
    public function query($query)
    {
        $this->reset();
        $statement = DB::connection()
                ->getPdo()
                ->prepare($query);
        $statement->execute();

        $this->result = $statement;
        $this->error = $statement->errorInfo(); 
        if ($this->error[0] != '00000')
        {
            $callers=debug_backtrace();
            $caller = isset($callers[0])?$callers[0]:array();
            $backtrace_html = '';
            if (isset($caller['file'])) { $backtrace_html .= 'File '.$caller['file'].(isset($caller['line'])?' ['.$caller['line'].']':'').'; '; }
            if (isset($caller['class'])) { $backtrace_html .= 'Class '.$caller['class'].'; '; }
            if (isset($caller['function'])) { $backtrace_html .= 'Function '.$caller['function'].'; '; }
            
            throw new Exception('MYSQL QUERY ERROR: '.$this->error.'<br />'.$backtrace_html);
        }
        $this->affected_rows = $statement->rowCount(); //$this->num_rows($statement);
        $this->last_insert_id = $this->insert_id();
        return $this;
    }   
    
    /** 
     * FUNCTION real_escape_string 
     * Function mysql_real_escape_string has been removed in PHP7, this function uses PDO.
     */
    public function real_escape_string($string)
    {  
        return substr(DB::connection()->getPdo()->quote($string),1,-1);
    }
    
    /** 
     * FUNCTION insert_id 
     * Function mysql_insert_id has been removed in PHP7, this function uses PDO.
     */
    public function insert_id()
    {
        return DB::connection()
                ->getPdo()
                ->lastInsertId();
    }
    
    /** 
     * FUNCTION fetch_assoc 
     * Function mysql_fetch_assoc has been removed in PHP7, this function uses PDO.
     */
    public static function fetch_assoc($query)
    {
        return $query->fetch(\PDO::FETCH_ASSOC);
    }
    
    /** 
     * FUNCTION safeQuery 
     * Query database by executing $query with replaced '?' characters by their equivalents from $params array;
     * @param string $query - a query to execute; all '?' will be replaced 
     * @param array $params - [optional] an array with parameters which are replacing '?' character
     * @return object An object of this class 
     */
    public function safeQuery($query,$params=false)
    {
        if ($params)
        {
            foreach ($params as &$v) { $v = $this->escape($v); }
            $query = vsprintf( str_replace('?','%s',$query), $params );
        }
        
        return $this->query($query);
    }
    
    
    /** 
     * FUNCTION escape 
     * Escapes and quotes strings for database query
     * @param string $param - a string to escape and quote
     * @return string An escaped and quoted string
     */
    public function escape($param)
    {
        return $this->quote($this->escapeNoQuotes($param));
    }
    
    
    /** 
     * FUNCTION escapeNoQuotes 
     * Escapes strings for database query
     * @param string $param - a string to escape
     * @return string An escaped string
     */
    public function escapeNoQuotes($param)
    {
        return $this->real_escape_string($param);
    }
    
    
    /** 
     * FUNCTION escapeLike 
     * Escapes and quotes strings for database query; additionaly escapes all '%' and '_' characters
     * @param string $param - a string to escape and quote
     * @return string An escaped and quoted string
     */
    public function escapeLike($param)
    {
        return $this->quote(escapeLikeNoQuotes($param));
    }
    
    
    /** 
     * FUNCTION escapeLikeNoQuotes 
     * Escapes strings for database query; additionaly escapes all '%' and '_' characters
     * @param string $param - a string to escape
     * @return string An escaped string
     */
    public function escapeLikeNoQuotes($param)
    {
        return str_replace(array('%','_'),array('\%','\_'),$this->escapeNoQuotes($param));
    }
    
    
    /** 
     * FUNCTION quote 
     * Quotes strings for database query
     * @param string $param - a string to quote
     * @return string A quoted string
     */
    public function quote($param)
    {
        return '"'.$param.'"';
    }
        
    
    /** 
     * FUNCTION asArray 
     * Returns array representing results of last query
     * @param string $key - [optional] a name of database column that were returned in last query and is the key of the returned array rows 
     */
    public function asArray($key = false, $onlycolumnvalue = false)
    {
        $result = $this->result;
        $array = array();
        //if (!is_resource($result)) { return $array; }
        while($data = self::fetch_assoc($result))
        {
            if($key !== false)
            {
                $array[$data[$key]] = ($onlycolumnvalue!==false?$data[$onlycolumnvalue]:$data);
            }
            else
            {
                $array[] = ($onlycolumnvalue!==false?$data[$onlycolumnvalue]:$data);
            }
        }
        return $array;
    }
    
    
    /** 
     * FUNCTION get 
     * Returns array representing first result of last query
     * @param string $name - [optional] a name of database column that were returned in last query; if specified the returned value qould be value of this column rather than whole row
     */
    public function get($name = false)
    {
        $array = $this->asArray();
        $result = FALSE;
        if ($name !== false)
        {
            if (isset($array[0][$name])) { $result = $array[0][$name]; }
        }
        else
        {
            if (isset($array[0])) { $result = $array[0]; }
        }
        return $result;
    }
    
    
    /** 
     * FUNCTION get 
     * Returns number of rows affected in last query
     */
    public function affectedRows()
    {
        return $this->affected_rows;
    }
    
    
    /** 
     * FUNCTION lastInsertID 
     * Returns last insert id of last query
     */
    public function lastInsertID()
    {
        return $this->last_insert_id;
    }
    
    
    /** 
     * FUNCTION lastInsertID 
     * Returns error message of last query
     */
    public function lastError()
    {
        return $this->error;
    }
        
        
}